#ifndef __PIPE_H
#define __PIPE_H

#include "list.h"

#define PIPE_RD 1
#define PIPE_WR 2

struct buffer;

#ifndef SWIG

struct pipe
{
  struct link k; /* owned by a task */
  struct buffer*b;
  struct pipe*next; /* next pipe on this buffer */
  unsigned int mode;
  int i; /* index into buffer mem */
  int i_lim; /* stop here, neg is infinity */
  struct pipe*_next;  /* used in composite.c */
  int id;
};
#endif

typedef struct pipe*PIPE; 
typedef struct pipe*WRITER; 
typedef struct pipe*READER; 

struct pipe*
pipe_alloc();
void
pipe_init(struct pipe*p,/*struct task*,*/int mode);
struct pipe*
pipe_new(/*struct task*,*/int mode);
void
pipe_attach(struct pipe*p,struct buffer*);
void
pipe_detach(struct pipe*p);
struct pipe*
reader_new(struct buffer*b);
struct pipe*
writer_new(struct buffer*b);
void 
pipe_reset(struct pipe*p);
char*
pipe_str(struct pipe*p);
void
pipe_print(struct pipe*p,int);
int
pipe_id(struct pipe*p);

unsigned int
read_size(struct pipe*p);
unsigned int
write_size(struct pipe*p);

void
produce(struct pipe*p,int i);
void
consume(struct pipe*p,int i);

int
is_reader(struct pipe*p);
int
is_writer(struct pipe*p);

void*
reader_mem(struct pipe*p);
void*
writer_mem(struct pipe*p);

int
reader_request(struct pipe*p,int size);
int
writer_request(struct pipe*p,int size);

int
read_msg(struct pipe*p,char*c,int n);

#define MSG_LEN 128
int
write_msg(struct pipe*p,char*c);

void
reader_seek(struct pipe*p,int i);
void
writer_seek(struct pipe*p,int i);

void
pipe_limit(struct pipe*p, int i);
int
pipe_get_limit(struct pipe*p);
void
pipe_no_limit(struct pipe*p);
int
pipe_get_i(struct pipe*p);
int
pipe_done(struct pipe*p);

void
_writer_nudge(struct pipe*p,int i);
void
_reader_nudge(struct pipe*p,int i);

void 
pipe_free(struct pipe*);

#endif

